# coding: utf-8

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os, re, subprocess, logging

def create_sourcearchive(baseDir, archivePath):
  exclude="--exclude '.*' --exclude devenv --exclude '*.pyc' --exclude '*.dylib' --exclude '.DS_Store'"
  subprocess.check_call('tar zcvf {0} {1} * 2>/dev/null'.format(archivePath, exclude), shell=True, cwd=baseDir)

def run(baseDir, type, version, downloadsRepo):  
  import buildtools.packagerChrome as packager

  # Replace version number in metadata file "manually", ConfigParser will mess
  # up the order of lines.
  metadata = packager.readMetadata(baseDir, type)
  with open(metadata.option_source("general", "version"), 'r+b') as file:
    rawMetadata = file.read()
    rawMetadata = re.sub(
      r'^(\s*version\s*=\s*).*', r'\g<1>%s' % version,
      rawMetadata, flags=re.I | re.M
    )

    file.seek(0)
    file.write(rawMetadata)
    file.truncate()

  # Read extension name from locale data
  import buildtools.packagerGecko as packagerGecko
  if type == "gecko":
    locales_base = baseDir
  else:
    # This is somewhat of a hack but reading out locale import config here would be too much
    locales_base = os.path.join(baseDir, "adblockplus")

  locales = packagerGecko.readLocaleMetadata(locales_base, [packagerGecko.defaultLocale])
  extensionName = locales[packagerGecko.defaultLocale]['name']

  # We actually have to create a unsigned Chrome build for Chrome Web Store
  metadata = packager.readMetadata(baseDir, "chrome")
  buildPathUnsigned = os.path.join(baseDir, downloadsRepo, packager.getDefaultFileName(metadata, version, 'zip'))
  packager.createBuild(baseDir, type=type, outFile=buildPathUnsigned, releaseBuild=True, keyFile=None)
  
  # Create source archive
  archivePath = os.path.splitext(buildPathUnsigned)[0] + '-source.tgz'
  create_sourcearchive(baseDir, archivePath)