/*
 * This file is part of MyAdFilter <https://www.myadfilter.com/>,
 * Copyright (C) 2015-2016 DYP Solutions
 *
 * MyAdFilter is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * MyAdFilter is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Adblock Plus.  If not, see <http://www.gnu.org/licenses/>.
 */
(function()
{
  var MyAdFilter = ext.MyAdFilter = function() {

    this.MYAF_DEBUG_ENABLE = false;

    this.dataAttribute = 'data-maf';
    this.dataAttrShow = 'show';
    this.dataAttrHide = 'hide';

    /**
     * rules to follow for the current page
     * @see observeWindowForCollapseElements
     */
    this.rules      = null;


    this.styleSheet = null;
    this.cssPrefix  = '';
    this.useAddRule = false;
    this.numberOfInsertedCssRules = 0;

    this._console = null;
  };

  MyAdFilter.prototype = {
    init: function(/**Window */ wnd, /**Integer*/ tabId) {
      if (wnd == wnd.top) {
        this._window = wnd;
        this._numberOfAds = 0;
        this.initCssClassNames();
      }
    },

    /**
     * Get CSS code to insert in to user StyleSheet
     *
     * @return array
     */
    getCss: function() {
      if (this.MYAF_DEBUG_ENABLE == true)
        return this.getColorCssRules();
      return this.getCssRules();
    },

    /**
     * Return the CSS stylesheet to insert for colorising element
     *
     * @return array
     */
    getColorCssRules: function() {
      return [
        { selector: '['+this.dataAttribute+'*="'+this.dataAttrHide+'"]' , rule: 'display: none !important;'                        },
        { selector: '['+this.dataAttribute+'*="'+this.dataAttrShow+'"]' , rule: 'box-shadow: inset 0px 0px 5px 2px green,  0px 0px 5px 2px green !important;' }
      ];
    },

    /**
     * Return the CSS stylesheet to insert for hiding elements
     *
     * @return array
     */
    getCssRules: function()
    {
      return [
        { selector: '['+this.dataAttribute+'*="'+this.dataAttrHide+'"]' , rule: 'display: none !important;'                        },
        { selector: '['+this.dataAttribute+'*="'+this.dataAttrShow+'"]' , rule: '' }
      ];
    },

    acceptThisAdNode: function(/**Node*/ node)
    {
      // APPLY policy to the node
      node.setAttribute(this.dataAttribute, this.dataAttrShow);

      // find rejected children -> accept them now
      var children = node.querySelectorAll('['+this.dataAttribute+'="'+this.dataAttrHide+'"]');
      for (var i = 0; i < children.length; i++) {
        children.setAttribute(this.dataAttribute, this.dataAttrShow);
      }

      // looking for rejected parent nodes -> accept them now
      var pnode = node;
      while (pnode = pnode.parentNode) {
        if (!('matches' in pnode))
          break;
        if (pnode.matches('['+this.dataAttribute+'="'+this.dataAttrHide+'"]'))
          pnode.setAttribute(this.dataAttribute, this.dataAttrShow); // accept this node
      }

      ext.backgroundPage.sendMessage({type: 'limitedsite-accepted-ad-hitcount'});
    },

    rejectThisAdNode: function(/**Node*/ node)
    {
      // node has accepted children
      if (node.querySelector('['+this.dataAttribute+'="'+this.dataAttrShow+'"]'))
        return;

      var pnode = node;
      while (pnode = pnode.parentNode) {
        if (!('matches' in pnode))
          break;
        if (pnode.matches('['+this.dataAttribute+'="'+this.dataAttrShow+'"]'))
          return;
      }

      // no children or parent accepted node. reject this node !
      node.setAttribute(this.dataAttribute, this.dataAttrHide);

      ext.backgroundPage.sendMessage({type: 'limitedsite-filter-hitcount'});
    },

    getParentNode: function(/**Node*/ node, /**String*/ parentSelector) {
      var rules = parentSelector.split('.');
      var pattern = /\.?(\w+)\((.*?)\)/;
      var up = 0;
      var maxUp = 30; // default max up


      rules = parentSelector.match(/\.?(\w+)\((.*?)\)/g);
      for (var r of rules) {
        var m = r.match(/^\.?(\w+)\((.*?)\)$/);
        var action = m[1];
        var param  = m[2];
        switch (action.toLowerCase()) {
          case 'maxup':
            if (param.match(/^[0-9]+$/))
              maxUp = parseInt(param);
            break;
          
          case 'parent':
            while (node = node.parentNode) {
              up++;
              if ('matches' in node && node.matches(param))
                break;
              if (up > maxUp)
                return null;
            }
            if (!node)
              return null;
            break;

          case 'prevSibling':
          case 'nextSibling':
            node = action == 'prevSibling' ? node.previousSibling : node.nextSibling;
            if (param != '' && ('matches' in node && !node.matches(param)))
              return null;
            break;

          case 'commentbefore':
          case 'commentafter':
            var cnode = action == 'commentbefore' ? node.previousSibling : node.nextSibling;
            if (cnode.nodeName != '#comment')
              return null;
            if (param != '' && cnode.textContent.toLowerCase().indexOf(param.toLowerCase()) == -1)
              return null;
            break;

          default: // nothing
            break;
        } // end of switch
      } // end of for

      return node;
    },

    applyCssRule: function(cssSelector, cssRule) {
      if (this.styleSheet) {
        this.log('applyCss : '+cssSelector+' { '+ cssRule +' }');
        if (this.useAddRule) {
          this.styleSheet.addRule(cssSelector, cssRule)
        } else {
          this.styleSheet.insertRule(this.cssPrefix + cssSelector + '{' + cssRule + '}', this.numberOfInsertedCssRules);
          this.numberOfInsertedCssRules++;
        }
      }
    },

    _insertDomainCssRules: function() {
      /** insert generic css rules **/
      var myAFCssRules = this.getCss();
      for (var i = 0; i < myAFCssRules.length; i++) {
        this.applyCssRule(myAFCssRules[i].selector, myAFCssRules[i].rule);
      }

      /** Insert css rules defined in filters */
      for (var cssSelector in this.rules.domainCssRules)
        this.applyCssRule(cssSelector, this.rules.domainCssRules[ cssSelector ]);
    },

    _prepareRules: function() {
      /** transform rules to be more efficient **/
      this.rules.simpleSelectorsString = '';
      for (var selector in this.rules.simpleSelectors) {
        sel = selector + ':not(['+this.dataAttribute+'])';
        this.rules.simpleSelectorsString += (this.rules.simpleSelectorsString == '' ? sel : ','+sel);
      }
      var newComplexSelectors = Object.create(null);
      for (var selector in this.rules.complexSelectors) {
        sel = selector +':not(['+this.dataAttribute+'])';
        newComplexSelectors[ sel ] = this.rules.complexSelectors[ selector ];
      }
      this.rules.complexSelectors = newComplexSelectors;
    },

    /**
     * Callback of mutation observer
     */
    _checkForCssCollapseElement: function(/**MutationRecord*/ mutations, observer) {
      /* simple selector's rules */
      var nodes = document.querySelectorAll(this.rules.simpleSelectorsString);
      for (var i = 0; i < nodes.length; i++)
        this.rejectThisAdNode(nodes[i]);

      /* complex selector's rules */
      for (selector in this.rules.complexSelectors) {
        var nodes = document.querySelectorAll(selector);
        if (nodes.length > 0) {
          var parentSelector = this.rules.complexSelectors[ selector ];
          this.log('match - '+selector+' - '+parentSelector);
          if (parentSelector) {
            for (var i = 0; i < nodes.length; i++) {
              var nn = null;
              // looking for the parent
              if (nn = this.getParentNode(nodes[i], parentSelector)) {
                this.rejectThisAdNode(nodes[i]);
                this.rejectThisAdNode(nn);
              }
              // if a parent node is rejected, reject this node
              // to prevent infinte selection of this node
              var pnode = nodes[i];
              while (pnode = pnode.parentNode) {
                if (!('matches' in pnode))
                  break;
                if (pnode.matches('['+this.dataAttribute+'="'+this.dataAttrHide+'"]')) {
                  nodes[i].setAttribute(this.dataAttribute, this.dataAttrHide);
                  break;
                }
              }
            }
          } else {
            for (var i = 0; i < nodes.length; i++)
              this.rejectThisAdNode(nodes[i]);
          }
        }
      }
    },

    /**
     * Check css selector if the dom was motified
     */
    observeWindowForCollapseElements: function(/**Window*/ wnd, /**Object*/ rules, /**StyleSheet*/ styleSheet, /**String*/ cssPrefix) {
      // only observe the main frame window
      if (wnd.top != wnd)
        return;

      // if we have already put an observe, go away
      if ('myCollapseObserver' in wnd)
        return;
      /* keep context */
      this.rules      = rules;
      this.styleSheet = styleSheet;
      this.cssPrefix  = cssPrefix;
      this.numberOfInsertedCssRules = 0;

      this._insertDomainCssRules();
      this._prepareRules();

      MutationObserver = wnd.MutationObserver || wnd.WebKitMutationObserver;
      wnd.myCollapseObserver = new MutationObserver(this._checkForCssCollapseElement.bind(this));

      var observeWnd = function(event) {
        // source: https://developer.mozilla.org/en-US/docs/Web/API/MutationObserver
        // NOTE: At the very least, childList, attributes, or characterData must be set to true.
        // Otherwise, "An invalid or illegal string was specified" error is thrown.
        try {
          wnd.myCollapseObserver.observe(wnd.document, {
            subtree: true,
            childList: true,
            attributes: true,
            characterData: false
          });
        } catch(e) {
          // Got this exception, I don't know why :( :(
          // only when called directly.
          // TypeError: The expression cannot be converted to return the specified type
        }
      };

      this.doAfterWindowLoaded(wnd, observeWnd);

      wnd.addEventListener("unload", function(event) {
        if ('myCollapseObserver' in this && this.myCollapseObserver) {
          this.myCollapseObserver.disconnect();
          this.myCollapseObserver = false;
          delete this.myCollapseObserver;
        }
      }, false);
    },

    /**
     * Execute a callback when the document in loaded.
     *
     * @param Window wnd
     * @param callback callback
     */
    doAfterWindowLoaded: function(/**Window*/ wnd, callback) {
      if (wnd.document.readyState == "complete" 
         || wnd.document.readyState == "loaded" 
         || wnd.document.readyState == "interactive"
      ) {
        callback();
      } else {
        wnd.addEventListener('DOMContentLoaded', function() {
          callback();
          wnd.removeEventListener('DOMContentLoaded', arguments.callee);
        }, true);
      }
    },

    /**
     * @param string message
     */
    log: function(/**String*/ message)
    {
      if (!this.MYAF_DEBUG_ENABLE)
        return;
      if (!this._console) {
        this._console = console;
      }
      this._console.log(message);
    }
  };
})();
